#!/usr/bin/env python

"""
Common classes.

Copyright (C) 2007, 2008 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

try:
    set
except NameError:
    from sets import Set as set

# Errors.

class ProcessingError(Exception):

    "A processing error."

    pass

class InspectError(ProcessingError):

    "An error during the module inspection process."

    pass

class TableError(ProcessingError):

    "An error occurring during access to a lookup table."

    pass

class NodeProcessingError(ProcessingError):

    "A processing error associated with a particular program node."

    def __init__(self, unit_name, node, message):
        self.unit_name = unit_name
        self.astnode = node
        self.message = message

    def __repr__(self):
        return "Error in %r at line %d: %s" % (self.unit_name, self.astnode.lineno, self.message)

    def __str__(self):
        return repr(self)

class TranslateError(NodeProcessingError):

    "An error during the module translation process."

    pass

class TranslationNotImplementedError(TranslateError):

    "An error caused by a node not being supported in translation."

    pass

# Program code representations.

class Block:

    "A code block."

    def __init__(self):
        self.code = []
        self.location = None

    def __repr__(self):
        return "Block(%r, location=%r)" % (id(self), self.location)

# Inspection representations.

class AtLeast:

    "A special representation for numbers of a given value or greater."

    def __init__(self, count):
        self.count = count

    def __eq__(self, other):
        return 0

    __lt__ = __le__ = __eq__

    def __ne__(self, other):
        return 1

    def __gt__(self, other):
        if isinstance(other, AtLeast):
            return 0
        else:
            return self.count > other

    def __ge__(self, other):
        if isinstance(other, AtLeast):
            return 0
        else:
            return self.count >= other

    def __iadd__(self, other):
        if isinstance(other, AtLeast):
            self.count += other.count
        else:
            self.count += other
        return self

    def __radd__(self, other):
        if isinstance(other, AtLeast):
            return AtLeast(self.count + other.count)
        else:
            return AtLeast(self.count + other)

    def __repr__(self):
        return "AtLeast(%r)" % self.count

class Naming:

    "A mix-in providing naming conveniences."

    def full_name(self):
        if self.name is not None:
            return self.parent.full_name() + "." + self.name
        else:
            return self.parent.full_name()

class Undefined:

    "A special class of undefined values."

    pass

# vim: tabstop=4 expandtab shiftwidth=4
